from django.shortcuts import render
from django.views import generic
from django.http import JsonResponse, HttpResponseRedirect, \
                        HttpResponseForbidden
from django.urls import reverse
from django.shortcuts import get_object_or_404
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User, Group
from django.utils.translation import gettext_lazy as _

from uhepp_api.models import Token

from .models import Plot, Collection

class PlotIndex(generic.ListView):
    template_name = 'uhepp_vault/plots.html'
    context_object_name = 'plots'

    def get_queryset(self):
        """Return all plots in the db"""
        return Plot.objects.all()

class PlotDetail(generic.DetailView):
    model = Plot
    template_name = 'uhepp_vault/plot.html'
    context_object_name = 'plot'

def plot_detail_json(request, uuid):
    plot = get_object_or_404(Plot, pk=uuid)

    return JsonResponse(plot.data)


def home(request):
    plot_count = Plot.objects.count()
    context = dict(plot_count=plot_count)
    return render(request, "uhepp_vault/home.html", context=context)

@login_required
def token_index(request):
    if request.method == "POST":
        form = TokenForm(request.POST)
        if form.is_valid():
            form_data = form.clean()
            new_token = Token.objects.create(user=request.user,
                                             description=form_data["description"],
                                             expires=form_data["expires"])
                                                      
            return HttpResponseRedirect(request.get_full_path())
    else:
        form = TokenForm()

    token_list = Token.objects.filter(user=request.user)

    context = {
        'user': request.user,
        'tokens': token_list,
        'token_form': form,
    }

    return render(request, "uhepp_vault/token.html", context=context)
    
@login_required
def account(request):

    collaborations = [g.collaboration
                      for g in request.user.groups.filter(collaboration__isnull=False)]

    context = {
        'user': request.user,
        'collaborations': collaborations
    }

    return render(request, "uhepp_vault/account.html", context=context)
    

def collection_create(request, username):
    user = get_object_or_404(User, username=username)
    if request.method == "POST":
        form = CollectionForm(request.POST)
        if form.is_valid():
            form_data = form.clean()

            collection = Collection.objects.create(user=user,
                                                   title=form_data["title"],
                                                   description=form_data["description"],
                                                   slug=form_data["slug"])
            assign_perm('uhepp_vault.view_collection', user, collection)
            assign_perm('uhepp_vault.discover_collection', user, collection)

            redirect_url = reverse('uhepp_vault:user-collection-detail',
                                    args=[username, collection.slug])
                                                      
            return HttpResponseRedirect(redirect_url)
    else:
        form = CollectionForm()

    context = {
        'form': form,
    }

    return render(request, "uhepp_vault/collection_create.html", context=context)

def user_detail(request, username):
    user = get_object_or_404(User, username=username)
    collaborations = [g.collaboration
                      for g in user.groups.filter(collaboration__isnull=False)]

    collections = get_objects_for_user(request.user, 'uhepp_vault.discover_collection',
                                       user.collections)

    collections = {c: c.plots.count() for c in collections.all()}

    context = dict(
        user=user,
        collaborations=collaborations,
        collections=collections
    )
    return render(request, "uhepp_vault/user_detail.html", context=context)

def collection_detail(request, pk):
    return _collection_detail(request, uuid=pk)

def user_collection_detail(request, username, slug):
    return _collection_detail(request, username=username, slug=slug)

def _collection_detail(request, uuid=None, username=None, slug=None):
    if uuid:
        collection = get_object_or_404(Collection, uuid=uuid)
    else:
        collection = get_object_or_404(Collection,
                                       slug=slug,
                                       user__username=username)

    collaborations = [g.collaboration
                      for g in collection.user.groups.filter(collaboration__isnull=False)]
    
    general_public = Group.objects.get(name="general-public")

    def mk_perm_form(perm, data=None):
        code_name = f"uhepp_vault.{perm}_collection"
        public_checker = ObjectPermissionChecker(general_public)
        options = []
        options.append((f"{perm}_general_public",
                        _("General public"), 
                        public_checker.has_perm(code_name, collection)))
        for collab in collaborations:
            # Add also other collabs who can view/discover this
            collab_checker = ObjectPermissionChecker(collab.perm_group)
            options.append((f"{perm}_collaboration_{collab.pk}",
                            str(collab),
                            collab_checker.has_perm(code_name, collection)))
        return CollectionPermForm(data, options=options)

    if request.method == "POST":
        access_perm_form = mk_perm_form("view", request.POST)
        discover_perm_form = mk_perm_form("discover", request.POST)

        if access_perm_form.is_valid() and discover_perm_form.is_valid():
            access_data = access_perm_form.clean()
            discover_data = discover_perm_form.clean()
            if access_data["view_general_public"]:
                assign_perm('uhepp_vault.view_collection',
                            general_public,
                            collection)
            else:
                remove_perm('uhepp_vault.view_collection',
                            general_public,
                            collection)
            if discover_data["discover_general_public"]:
                assign_perm('uhepp_vault.discover_collection',
                            general_public,
                            collection)
            else:
                remove_perm('uhepp_vault.discover_collection',
                            general_public,
                            collection)

            for collab in collaborations:
                # Add also other collabs who can view/discover this
                if access_data[f"view_collaboration_{collab.pk}"]:
                    assign_perm('uhepp_vault.view_collection',
                                collab.perm_group,
                                collection)
                else:
                    remove_perm('uhepp_vault.view_collection',
                                collab.perm_group,
                                collection)
                if discover_data[f"discover_collaboration_{collab.pk}"]:
                    assign_perm('uhepp_vault.discover_collection',
                                collab.perm_group,
                                collection)
                else:
                    remove_perm('uhepp_vault.discover_collection',
                                collab.perm_group,
                                collection)


    else:
        access_perm_form = mk_perm_form("view")
        discover_perm_form = mk_perm_form("discover")


    context = {
        'plots': collection.plots.all(),
        'owner': collection.user,
        'collection': collection,
        'access_perm_form': access_perm_form,
        'discover_perm_form': discover_perm_form,
    }

    return render(request, "uhepp_vault/collection.html", context=context)
